<?php

namespace RosreestrGeomToWKT;

use RosreestrGeomToWKT\WktUtilites;

class SpatialElement
{
    /**
    * @var array
    */
    private $spelementUnits;
    
    /**
    * @var object
    */
    private $ut;

    /**
    * @var string
    */
    public $spatialElementData;

    /**
    * @param array $spelementUnits
    */
    public function __construct(array $spelementUnits)
    {
        $this->spelementUnits = $spelementUnits;
        $this->ut = new WktUtilites();
        $this->spatialElementData = $this->getSpatialElementData();
    }

    /**
    * @return string
    * @throws
    */
    private function getSpatialElementData(): array
    {
        if ($this->ut::validateArray($this->spelementUnits)) {
            if (count($this->spelementUnits) === 1) {
                $spelementUnit = $this->spelementUnits[0];
                if (isset($spelementUnit->ordinate)) {
                    if ($this->ut::getValidCoordinates($spelementUnit->ordinate)) {
                        $coordinates = '(' . $spelementUnit->ordinate->x . ' ' . $spelementUnit->ordinate->y . ')';
                        
                        return ['type' => 'POINT', 'coordinates' => $coordinates];
                    } else {
                        throw new \Exception('Invalid coordinates');
                    }
                } else {
                    throw new \Exception('Undefined Ordinate');
                }
            } else {
                $firstEl = $this->spelementUnits[0];
                $lastEl = end($this->spelementUnits);

                $firstXY = $this->ut::getValidCoordinates($firstEl->ordinate);
                $lastXY = $this->ut::getValidCoordinates($lastEl->ordinate);

                if ($firstXY && $lastXY) {
                    if (count($this->spelementUnits) <= 3 && $firstXY == $lastXY) {
                        throw new \Exception('Not enough points for polygon');
                    } else {
                        $coordinates = $firstXY == $lastXY ? '((' : '(';
                        $elType = $firstXY == $lastXY ? 'POLYGON' : 'LINESTRING';

                        foreach ($this->spelementUnits as $spelementUnit) {
                            if (isset($spelementUnit->ordinate)) {
                                if ($this->ut::getValidCoordinates($spelementUnit->ordinate)) {
                                    $coordinates .= $spelementUnit->ordinate->x . ' ' . $spelementUnit->ordinate->y . ', ';
                                } else {
                                    throw new \Exception('Invalid coordinates');
                                    break;
                                }
                            } else {
                                throw new \Exception('Undefined Ordinate');
                                break;
                            }
                        }
                        $coordinates = rtrim($coordinates, ', ') . ($elType === 'POLYGON' ? '))' : ')');
                        return ['type' => $elType, 'coordinates' => $coordinates];
                    }
                } else {
                    throw new \Exception('Invalid coordinates of the first or(and) the last unit');
                }
            }
        } else {
            throw new \Exception('Spelement units are not found');
        }
    }
}
